// SPDX-FileCopyrightText: 2021 Paul Schaub <vanitasvitae@fsfe.org>
//
// SPDX-License-Identifier: Apache-2.0

/**
 * Stateless OpenPGP Interface for Java.
 * Exception classes.
 */
package sop.exception;
